# encoding: utf-8
#
# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2018 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# ** end header
#
"""Created on 27.10.2015.

@author: tf
"""

import pytest
import caosdb as h

''' some helpful functions '''
old_stored = None


def setup_function(function):
    try:
        h.execute_query("FIND ENTITY").delete()
    except BaseException:
        pass


def update_rec(rec, datetime):
    global old_stored
    if old_stored == datetime:
        return
    rec.get_property("SimpleDateProperty").value = datetime
    rec.update()
    old_stored = datetime


def store(rec, datetime):
    update_rec(rec, datetime)
    assert rec.is_valid()
    return h.Record(id=rec.id).retrieve().get_property(
        "SimpleDateProperty").value


def set_up_rec():
    global old_stored
    old_stored = None
    p = h.Property(name="SimpleDateProperty", datatype="DATETIME").insert()
    assert p.is_valid()

    rt = h.RecordType(
        name="SimpleRecordType").add_property(
        p, importance="OBLIGATORY").insert()
    assert rt.is_valid()

    rec = h.Record(name="SimpleRecord").add_parent(rt).add_property(p).insert()
    assert rec.is_valid()

    return rec


def query_assertion(rec, stored, op, queried):
    update_rec(rec, stored)
    assert rec.is_valid()

    q = h.execute_query(
        "FIND RECORD SimpleRecord WHICH HAS A SimpleDateProperty" +
        op +
        queried)
    return len(q) == 1


''' the actual tests '''


def test_utcdatetime_storage():
    rec = set_up_rec()

    utcs = [
        "2015-12-24T20:15:30",
        "2015-12-24T20:15:30.123",
        "-9999-01-01T00:00:00",
        "-9999-01-01T00:00:00.0",
        "9999-12-31T23:59:59",
        "9999-12-31T23:59:59.999999999",
        "0001-01-01T00:00:00",
        "0001-01-01T00:00:00.0",
        "-0001-12-31T23:59:59",
        "-0001-12-31T23:59:59.999999"]
    for d in utcs:
        assert d in store(rec, d)


def test_date_storage():
    rec = set_up_rec()

    dates = [
        "2015-12-24",
        "2015",
        "2015-12",
        "9999-12-31",
        "9999-12",
        "9999",
        "-9999-01-01",
        "-9999-01",
        "-9999",
        "0001-01-01",
        "0001-01",
        "0001",
        "-0001-12-31",
        "-0001-12",
        "-0001"]
    for d in dates:
        assert d in store(rec, d)


@pytest.mark.slow
def test_in_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert query_assertion(rec, d, " IN ", d)
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20:15")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20")
    assert query_assertion(rec, d, " IN ", "2015-05-05")
    assert query_assertion(rec, d, " IN ", "2015-05")
    assert query_assertion(rec, d, " IN ", "2015")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20")
    assert not query_assertion(rec, d, " IN ", "2015-05-04")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20")
    assert not query_assertion(rec, d, " IN ", "2015-04-05")
    assert not query_assertion(rec, d, " IN ", "2015-04")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20")
    assert not query_assertion(rec, d, " IN ", "2014-05-05")
    assert not query_assertion(rec, d, " IN ", "2014-05")
    assert not query_assertion(rec, d, " IN ", "2014")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert query_assertion(rec, d, " IN ", d)
    assert query_assertion(rec, d, " IN ", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20:15")
    assert query_assertion(rec, d, " IN ", "2015-05-05T20")
    assert query_assertion(rec, d, " IN ", "2015-05-05")
    assert query_assertion(rec, d, " IN ", "2015-05")
    assert query_assertion(rec, d, " IN ", "2015")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T20:16")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-05T21")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-04T20")
    assert not query_assertion(rec, d, " IN ", "2015-05-04")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-04-05T20")
    assert not query_assertion(rec, d, " IN ", "2015-04-05")
    assert not query_assertion(rec, d, " IN ", "2015-04")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20:15")
    assert not query_assertion(rec, d, " IN ", "2014-05-05T20")
    assert not query_assertion(rec, d, " IN ", "2014-05-05")
    assert not query_assertion(rec, d, " IN ", "2014-05")
    assert not query_assertion(rec, d, " IN ", "2014")

    # Date YYYY-MM-dd
    d = "2015-01-01"
    assert query_assertion(rec, d, " IN ", d)
    assert not query_assertion(rec, d, " IN ", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-01-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-01-01T20")
    assert query_assertion(rec, d, " IN ", "2015-01-01")
    assert query_assertion(rec, d, " IN ", "2015-01")
    assert query_assertion(rec, d, " IN ", "2015")
    assert not query_assertion(rec, d, " IN ", "2015-02-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-02-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-02-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-02-01T20")
    assert not query_assertion(rec, d, " IN ", "2015-02-01")
    assert not query_assertion(rec, d, " IN ", "2015-02")
    assert not query_assertion(rec, d, " IN ", "2016-01-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2016-01-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2016-01-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2016-01-01T20")
    assert not query_assertion(rec, d, " IN ", "2016-01-01")
    assert not query_assertion(rec, d, " IN ", "2016-01")
    assert not query_assertion(rec, d, " IN ", "2016")
    assert not query_assertion(rec, d, " IN ", "2014-01-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2014-01-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2014-01-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2014-01-01T20")
    assert not query_assertion(rec, d, " IN ", "2014-01-01")
    assert not query_assertion(rec, d, " IN ", "2014-01")
    assert not query_assertion(rec, d, " IN ", "2014")

    # Date YYYY-MM
    d = "2015-05"
    assert query_assertion(rec, d, " IN ", d)
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20")
    assert not query_assertion(rec, d, " IN ", "2015-05-01")
    assert query_assertion(rec, d, " IN ", "2015")
    assert not query_assertion(rec, d, " IN ", "2015-06-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-06-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-06-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-06-01T20")
    assert not query_assertion(rec, d, " IN ", "2015-06-01")
    assert not query_assertion(rec, d, " IN ", "2015-06")
    assert not query_assertion(rec, d, " IN ", "2015-04-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-04-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-04-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-04-01T20")
    assert not query_assertion(rec, d, " IN ", "2015-04-01")
    assert not query_assertion(rec, d, " IN ", "2015-04")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20")
    assert not query_assertion(rec, d, " IN ", "2016-05-01")
    assert not query_assertion(rec, d, " IN ", "2016-05")
    assert not query_assertion(rec, d, " IN ", "2016")

    # Date YYYY
    d = "2015"
    assert query_assertion(rec, d, " IN ", d)
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2015-05-01T20")
    assert not query_assertion(rec, d, " IN ", "2015-05-01")
    assert not query_assertion(rec, d, " IN ", "2015-05")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2016-05-01T20")
    assert not query_assertion(rec, d, " IN ", "2016-05-01")
    assert not query_assertion(rec, d, " IN ", "2016-05")
    assert not query_assertion(rec, d, " IN ", "2016")
    assert not query_assertion(rec, d, " IN ", "2014-05-01T20:15:30.123")
    assert not query_assertion(rec, d, " IN ", "2014-05-01T20:15:30")
    assert not query_assertion(rec, d, " IN ", "2014-05-01T20:15")
    assert not query_assertion(rec, d, " IN ", "2014-05-01T20")
    assert not query_assertion(rec, d, " IN ", "2014-05-01")
    assert not query_assertion(rec, d, " IN ", "2014-05")
    assert not query_assertion(rec, d, " IN ", "2014")


@pytest.mark.slow
def test_not_in_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert not query_assertion(rec, d, " NOT IN ", d)
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-06")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2016-05")
    assert query_assertion(rec, d, " NOT IN ", "2016")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert not query_assertion(rec, d, " NOT IN ", d)
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05T20")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:16")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T21:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-06")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-06")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2016-05")
    assert query_assertion(rec, d, " NOT IN ", "2016")

    # Date YYYY-MM-dd
    d = "2015-05-05"
    assert not query_assertion(rec, d, " NOT IN ", d)
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-06")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-04")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2016-05")
    assert query_assertion(rec, d, " NOT IN ", "2016")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2014-05")
    assert query_assertion(rec, d, " NOT IN ", "2014")

    # Date YYYY-MM
    d = "2015-05"
    assert not query_assertion(rec, d, " NOT IN ", d)
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-06-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-06")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-04-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-04")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2016-05")
    assert query_assertion(rec, d, " NOT IN ", "2016")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2014-05")
    assert query_assertion(rec, d, " NOT IN ", "2014")

    # Date YYYY
    d = "2015"
    assert not query_assertion(rec, d, " NOT IN ", d)
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2015-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2015-05")
    assert not query_assertion(rec, d, " NOT IN ", "2015")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2016-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2016-05")
    assert query_assertion(rec, d, " NOT IN ", "2016")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20:15")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05T20")
    assert query_assertion(rec, d, " NOT IN ", "2014-05-05")
    assert query_assertion(rec, d, " NOT IN ", "2014-05")
    assert query_assertion(rec, d, " NOT IN ", "2014")


@pytest.mark.slow
def test_eq_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert query_assertion(rec, d, "=", d)
    assert not query_assertion(rec, d, "=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "=", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "=", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "=", "2015-05-05T20")
    assert not query_assertion(rec, d, "=", "2015-05-05")
    assert not query_assertion(rec, d, "=", "2015-05")
    assert not query_assertion(rec, d, "=", "2015")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert query_assertion(rec, d, "=", d)
    assert query_assertion(rec, d, "=", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "=", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "=", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "=", "2015-05-05T20")
    assert not query_assertion(rec, d, "=", "2015-05-05")
    assert not query_assertion(rec, d, "=", "2015-05")
    assert not query_assertion(rec, d, "=", "2015")

    # Date YYYY-MM-dd
    d = "2015-01-01"
    assert query_assertion(rec, d, "=", d)
    assert not query_assertion(rec, d, "=", "2015-01-02")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "=", "2015-01-01T20")
    assert not query_assertion(rec, d, "=", "2015-01")
    assert not query_assertion(rec, d, "=", "2015")

    # Date YYYY-MM
    d = "2015-01"
    assert query_assertion(rec, d, "=", d)
    assert not query_assertion(rec, d, "=", "2015-02")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "=", "2015-01-01T20")
    assert not query_assertion(rec, d, "=", "2015-01-01")
    assert not query_assertion(rec, d, "=", "2015")

    # Date YYYY
    d = "2015"
    assert query_assertion(rec, d, "=", d)
    assert not query_assertion(rec, d, "=", "2016")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "=", "2015-01-01T20")
    assert not query_assertion(rec, d, "=", "2015-01-01")
    assert not query_assertion(rec, d, "=", "2015-01")


@pytest.mark.slow
def test_neq_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert not query_assertion(rec, d, "!=", d)
    assert query_assertion(rec, d, "!=", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20")
    assert not query_assertion(rec, d, "!=", "2015-05-05")
    assert not query_assertion(rec, d, "!=", "2015-05")
    assert not query_assertion(rec, d, "!=", "2015")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert not query_assertion(rec, d, "!=", d)
    assert query_assertion(rec, d, "!=", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "!=", "2015-05-05T20")
    assert not query_assertion(rec, d, "!=", "2015-05-05")
    assert not query_assertion(rec, d, "!=", "2015-05")
    assert not query_assertion(rec, d, "!=", "2015")

    # Date YYYY-MM-dd
    d = "2015-01-01"
    assert not query_assertion(rec, d, "!=", d)
    assert query_assertion(rec, d, "!=", "2015-01-02")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20")
    assert not query_assertion(rec, d, "!=", "2015-01")
    assert not query_assertion(rec, d, "!=", "2015")

    # Date YYYY-MM
    d = "2015-01"
    assert not query_assertion(rec, d, "!=", d)
    assert query_assertion(rec, d, "!=", "2015-02")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20")
    assert not query_assertion(rec, d, "!=", "2015-01-01")
    assert not query_assertion(rec, d, "!=", "2015")

    # Date YYYY
    d = "2015"
    assert not query_assertion(rec, d, "!=", d)
    assert query_assertion(rec, d, "!=", "2016")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30.123")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15:30")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20:15")
    assert not query_assertion(rec, d, "!=", "2015-01-01T20")
    assert not query_assertion(rec, d, "!=", "2015-01-01")
    assert not query_assertion(rec, d, "!=", "2015-01")


@pytest.mark.slow
def test_smaller_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert not query_assertion(rec, d, "<", d)
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T20")
    assert not query_assertion(rec, d, "<", "2015-05-05")
    assert not query_assertion(rec, d, "<", "2015-05")
    assert not query_assertion(rec, d, "<", "2015")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.122")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:29.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:29")
    assert query_assertion(rec, d, "<", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, "<", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T19")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15")
    assert query_assertion(rec, d, "<", "2015-05-05T21")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-04T20")
    assert not query_assertion(rec, d, "<", "2015-05-04")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<", "2015-05-06T20")
    assert query_assertion(rec, d, "<", "2015-05-06")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-04-05T20")
    assert not query_assertion(rec, d, "<", "2015-04-05")
    assert not query_assertion(rec, d, "<", "2015-04")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<", "2015-06-05T20")
    assert query_assertion(rec, d, "<", "2015-06-05")
    assert query_assertion(rec, d, "<", "2015-06")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2014-05-05T20")
    assert not query_assertion(rec, d, "<", "2014-05-05")
    assert not query_assertion(rec, d, "<", "2014-05")
    assert not query_assertion(rec, d, "<", "2014")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<", "2016-05-05T20")
    assert query_assertion(rec, d, "<", "2016-05-05")
    assert query_assertion(rec, d, "<", "2016-05")
    assert query_assertion(rec, d, "<", "2016")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert not query_assertion(rec, d, "<", d)
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T20")
    assert not query_assertion(rec, d, "<", "2015-05-05")
    assert not query_assertion(rec, d, "<", "2015-05")
    assert not query_assertion(rec, d, "<", "2015")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.122")
    assert query_assertion(rec, d, "<", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:29.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:29")
    assert query_assertion(rec, d, "<", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, "<", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:14")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, "<", "2015-05-05T20:16")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T19:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T19")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, "<", "2015-05-05T21:15")
    assert query_assertion(rec, d, "<", "2015-05-05T21")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-04T20")
    assert not query_assertion(rec, d, "<", "2015-05-04")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<", "2015-05-06T20")
    assert query_assertion(rec, d, "<", "2015-05-06")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-04-05T20")
    assert not query_assertion(rec, d, "<", "2015-04-05")
    assert not query_assertion(rec, d, "<", "2015-04")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<", "2015-06-05T20")
    assert query_assertion(rec, d, "<", "2015-06-05")
    assert query_assertion(rec, d, "<", "2015-06")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2014-05-05T20")
    assert not query_assertion(rec, d, "<", "2014-05-05")
    assert not query_assertion(rec, d, "<", "2014-05")
    assert not query_assertion(rec, d, "<", "2014")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<", "2016-05-05T20")
    assert query_assertion(rec, d, "<", "2016-05-05")
    assert query_assertion(rec, d, "<", "2016-05")
    assert query_assertion(rec, d, "<", "2016")

    # Date YYYY-MM-dd
    d = "2015-05-05"
    assert not query_assertion(rec, d, "<", d)
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T20")
    assert not query_assertion(rec, d, "<", "2015-05-05")
    assert not query_assertion(rec, d, "<", "2015-05")
    assert not query_assertion(rec, d, "<", "2015")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-04T20")
    assert not query_assertion(rec, d, "<", "2015-05-04")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<", "2015-05-06T20")
    assert query_assertion(rec, d, "<", "2015-05-06")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-04-05T20")
    assert not query_assertion(rec, d, "<", "2015-04-05")
    assert not query_assertion(rec, d, "<", "2015-04")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<", "2015-06-05T20")
    assert query_assertion(rec, d, "<", "2015-06-05")
    assert query_assertion(rec, d, "<", "2015-06")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2014-05-05T20")
    assert not query_assertion(rec, d, "<", "2014-05-05")
    assert not query_assertion(rec, d, "<", "2014-05")
    assert not query_assertion(rec, d, "<", "2014")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<", "2016-05-05T20")
    assert query_assertion(rec, d, "<", "2016-05-05")
    assert query_assertion(rec, d, "<", "2016-05")
    assert query_assertion(rec, d, "<", "2016")

    # Date YYYY-MM
    d = "2015-05"
    assert not query_assertion(rec, d, "<", d)
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T20")
    assert not query_assertion(rec, d, "<", "2015-05-05")
    assert not query_assertion(rec, d, "<", "2015-05")
    assert not query_assertion(rec, d, "<", "2015")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-04-05T20")
    assert not query_assertion(rec, d, "<", "2015-04-05")
    assert not query_assertion(rec, d, "<", "2015-04")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<", "2015-06-05T20")
    assert query_assertion(rec, d, "<", "2015-06-05")
    assert query_assertion(rec, d, "<", "2015-06")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2014-05-05T20")
    assert not query_assertion(rec, d, "<", "2014-05-05")
    assert not query_assertion(rec, d, "<", "2014-05")
    assert not query_assertion(rec, d, "<", "2014")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<", "2016-05-05T20")
    assert query_assertion(rec, d, "<", "2016-05-05")
    assert query_assertion(rec, d, "<", "2016-05")
    assert query_assertion(rec, d, "<", "2016")

    # Date YYYY
    d = "2015"
    assert not query_assertion(rec, d, "<", d)
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2015-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2015-05-05T20")
    assert not query_assertion(rec, d, "<", "2015-05-05")
    assert not query_assertion(rec, d, "<", "2015-05")
    assert not query_assertion(rec, d, "<", "2015")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<", "2014-05-05T20")
    assert not query_assertion(rec, d, "<", "2014-05-05")
    assert not query_assertion(rec, d, "<", "2014-05")
    assert not query_assertion(rec, d, "<", "2014")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<", "2016-05-05T20")
    assert query_assertion(rec, d, "<", "2016-05-05")
    assert query_assertion(rec, d, "<", "2016-05")
    assert query_assertion(rec, d, "<", "2016")


@pytest.mark.slow
def test_greater_operator():
    rec = set_up_rec()
    # query with >
    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert not query_assertion(rec, d, ">", d)
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T20")
    assert not query_assertion(rec, d, ">", "2015-05-05")
    assert not query_assertion(rec, d, ">", "2015-05")
    assert not query_assertion(rec, d, ">", "2015")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.122")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, ">", "2015-05-05T20:15:29.123")
    assert query_assertion(rec, d, ">", "2015-05-05T20:15:29")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14:30.123")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14:30")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15:30.123")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15:30")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15")
    assert query_assertion(rec, d, ">", "2015-05-05T19")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T21")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">", "2015-05-04T20")
    assert query_assertion(rec, d, ">", "2015-05-04")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-06T20")
    assert not query_assertion(rec, d, ">", "2015-05-06")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">", "2015-04-05T20")
    assert query_assertion(rec, d, ">", "2015-04-05")
    assert query_assertion(rec, d, ">", "2015-04")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-06-05T20")
    assert not query_assertion(rec, d, ">", "2015-06-05")
    assert not query_assertion(rec, d, ">", "2015-06")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">", "2014-05-05T20")
    assert query_assertion(rec, d, ">", "2014-05-05")
    assert query_assertion(rec, d, ">", "2014-05")
    assert query_assertion(rec, d, ">", "2014")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2016-05-05T20")
    assert not query_assertion(rec, d, ">", "2016-05-05")
    assert not query_assertion(rec, d, ">", "2016-05")
    assert not query_assertion(rec, d, ">", "2016")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert not query_assertion(rec, d, ">", d)
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T20")
    assert not query_assertion(rec, d, ">", "2015-05-05")
    assert not query_assertion(rec, d, ">", "2015-05")
    assert not query_assertion(rec, d, ">", "2015")
    assert query_assertion(rec, d, ">", "2015-05-05T20:15:30.122")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, ">", "2015-05-05T20:15:29.123")
    assert query_assertion(rec, d, ">", "2015-05-05T20:15:29")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14:30.123")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14:30")
    assert query_assertion(rec, d, ">", "2015-05-05T20:14")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:16")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15:30.123")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15:30")
    assert query_assertion(rec, d, ">", "2015-05-05T19:15")
    assert query_assertion(rec, d, ">", "2015-05-05T19")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T21:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T21")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">", "2015-05-04T20")
    assert query_assertion(rec, d, ">", "2015-05-04")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-06T20")
    assert not query_assertion(rec, d, ">", "2015-05-06")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">", "2015-04-05T20")
    assert query_assertion(rec, d, ">", "2015-04-05")
    assert query_assertion(rec, d, ">", "2015-04")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-06-05T20")
    assert not query_assertion(rec, d, ">", "2015-06-05")
    assert not query_assertion(rec, d, ">", "2015-06")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">", "2014-05-05T20")
    assert query_assertion(rec, d, ">", "2014-05-05")
    assert query_assertion(rec, d, ">", "2014-05")
    assert query_assertion(rec, d, ">", "2014")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2016-05-05T20")
    assert not query_assertion(rec, d, ">", "2016-05-05")
    assert not query_assertion(rec, d, ">", "2016-05")
    assert not query_assertion(rec, d, ">", "2016")

    # Date YYYY-MM-dd
    d = "2015-05-05"
    assert not query_assertion(rec, d, ">", d)
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T20")
    assert not query_assertion(rec, d, ">", "2015-05-05")
    assert not query_assertion(rec, d, ">", "2015-05")
    assert not query_assertion(rec, d, ">", "2015")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-06T20")
    assert not query_assertion(rec, d, ">", "2015-05-06")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">", "2015-05-04T20")
    assert query_assertion(rec, d, ">", "2015-05-04")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-06-05T20")
    assert not query_assertion(rec, d, ">", "2015-06-05")
    assert not query_assertion(rec, d, ">", "2015-06")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">", "2015-04-05T20")
    assert query_assertion(rec, d, ">", "2015-04-05")
    assert query_assertion(rec, d, ">", "2015-04")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2016-05-05T20")
    assert not query_assertion(rec, d, ">", "2016-05-05")
    assert not query_assertion(rec, d, ">", "2016-05")
    assert not query_assertion(rec, d, ">", "2016")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">", "2014-05-05T20")
    assert query_assertion(rec, d, ">", "2014-05-05")
    assert query_assertion(rec, d, ">", "2014-05")
    assert query_assertion(rec, d, ">", "2014")

    # Date YYYY-MM
    d = "2015-05"
    assert not query_assertion(rec, d, ">", d)
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T20")
    assert not query_assertion(rec, d, ">", "2015-05-05")
    assert not query_assertion(rec, d, ">", "2015-05")
    assert not query_assertion(rec, d, ">", "2015")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-06-05T20")
    assert not query_assertion(rec, d, ">", "2015-06-05")
    assert not query_assertion(rec, d, ">", "2015-06")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">", "2015-04-05T20")
    assert query_assertion(rec, d, ">", "2015-04-05")
    assert query_assertion(rec, d, ">", "2015-04")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2016-05-05T20")
    assert not query_assertion(rec, d, ">", "2016-05-05")
    assert not query_assertion(rec, d, ">", "2016-05")
    assert not query_assertion(rec, d, ">", "2016")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">", "2014-05-05T20")
    assert query_assertion(rec, d, ">", "2014-05-05")
    assert query_assertion(rec, d, ">", "2014-05")
    assert query_assertion(rec, d, ">", "2014")

    # Date YYYY
    d = "2015"
    assert not query_assertion(rec, d, ">", d)
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2015-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2015-05-05T20")
    assert not query_assertion(rec, d, ">", "2015-05-05")
    assert not query_assertion(rec, d, ">", "2015-05")
    assert not query_assertion(rec, d, ">", "2015")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">", "2016-05-05T20")
    assert not query_assertion(rec, d, ">", "2016-05-05")
    assert not query_assertion(rec, d, ">", "2016-05")
    assert not query_assertion(rec, d, ">", "2016")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">", "2014-05-05T20")
    assert query_assertion(rec, d, ">", "2014-05-05")
    assert query_assertion(rec, d, ">", "2014-05")
    assert query_assertion(rec, d, ">", "2014")


@pytest.mark.slow
def test_greater_equal_operator():
    rec = set_up_rec()
    # query with > Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert query_assertion(rec, d, ">=", d)
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T20")
    assert query_assertion(rec, d, ">=", "2015-05-05")
    assert query_assertion(rec, d, ">=", "2015-05")
    assert query_assertion(rec, d, ">=", "2015")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.122")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:29.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:29")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T19")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-04T20")
    assert query_assertion(rec, d, ">=", "2015-05-04")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20")
    assert not query_assertion(rec, d, ">=", "2015-05-06")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-04-05T20")
    assert query_assertion(rec, d, ">=", "2015-04-05")
    assert query_assertion(rec, d, ">=", "2015-04")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20")
    assert not query_assertion(rec, d, ">=", "2015-06-05")
    assert not query_assertion(rec, d, ">=", "2015-06")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2014-05-05T20")
    assert query_assertion(rec, d, ">=", "2014-05-05")
    assert query_assertion(rec, d, ">=", "2014-05")
    assert query_assertion(rec, d, ">=", "2014")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20")
    assert not query_assertion(rec, d, ">=", "2016-05-05")
    assert not query_assertion(rec, d, ">=", "2016-05")
    assert not query_assertion(rec, d, ">=", "2016")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert query_assertion(rec, d, ">=", d)
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T20")
    assert query_assertion(rec, d, ">=", "2015-05-05")
    assert query_assertion(rec, d, ">=", "2015-05")
    assert query_assertion(rec, d, ">=", "2015")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.122")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:29.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:29")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:15:31.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:15:31")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:14")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16:30")
    assert not query_assertion(rec, d, ">=", "2015-05-05T20:16")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T19:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T19")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15:30")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21:15")
    assert not query_assertion(rec, d, ">=", "2015-05-05T21")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-04T20")
    assert query_assertion(rec, d, ">=", "2015-05-04")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20")
    assert not query_assertion(rec, d, ">=", "2015-05-06")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-04-05T20")
    assert query_assertion(rec, d, ">=", "2015-04-05")
    assert query_assertion(rec, d, ">=", "2015-04")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20")
    assert not query_assertion(rec, d, ">=", "2015-06-05")
    assert not query_assertion(rec, d, ">=", "2015-06")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2014-05-05T20")
    assert query_assertion(rec, d, ">=", "2014-05-05")
    assert query_assertion(rec, d, ">=", "2014-05")
    assert query_assertion(rec, d, ">=", "2014")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20")
    assert not query_assertion(rec, d, ">=", "2016-05-05")
    assert not query_assertion(rec, d, ">=", "2016-05")
    assert not query_assertion(rec, d, ">=", "2016")

    # Date YYYY-MM-dd
    d = "2015-05-05"
    assert query_assertion(rec, d, ">=", d)
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T20")
    assert query_assertion(rec, d, ">=", "2015-05-05")
    assert query_assertion(rec, d, ">=", "2015-05")
    assert query_assertion(rec, d, ">=", "2015")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20:15")
    assert not query_assertion(rec, d, ">=", "2015-05-06T20")
    assert not query_assertion(rec, d, ">=", "2015-05-06")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-04T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-04T20")
    assert query_assertion(rec, d, ">=", "2015-05-04")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20")
    assert not query_assertion(rec, d, ">=", "2015-06-05")
    assert not query_assertion(rec, d, ">=", "2015-06")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-04-05T20")
    assert query_assertion(rec, d, ">=", "2015-04-05")
    assert query_assertion(rec, d, ">=", "2015-04")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20")
    assert not query_assertion(rec, d, ">=", "2016-05-05")
    assert not query_assertion(rec, d, ">=", "2016-05")
    assert not query_assertion(rec, d, ">=", "2016")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2014-05-05T20")
    assert query_assertion(rec, d, ">=", "2014-05-05")
    assert query_assertion(rec, d, ">=", "2014-05")
    assert query_assertion(rec, d, ">=", "2014")

    # Date YYYY-MM
    d = "2015-05"
    assert query_assertion(rec, d, ">=", d)
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T20")
    assert query_assertion(rec, d, ">=", "2015-05-05")
    assert query_assertion(rec, d, ">=", "2015-05")
    assert query_assertion(rec, d, ">=", "2015")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20:15")
    assert not query_assertion(rec, d, ">=", "2015-06-05T20")
    assert not query_assertion(rec, d, ">=", "2015-06-05")
    assert not query_assertion(rec, d, ">=", "2015-06")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-04-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-04-05T20")
    assert query_assertion(rec, d, ">=", "2015-04-05")
    assert query_assertion(rec, d, ">=", "2015-04")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20")
    assert not query_assertion(rec, d, ">=", "2016-05-05")
    assert not query_assertion(rec, d, ">=", "2016-05")
    assert not query_assertion(rec, d, ">=", "2016")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2014-05-05T20")
    assert query_assertion(rec, d, ">=", "2014-05-05")
    assert query_assertion(rec, d, ">=", "2014-05")
    assert query_assertion(rec, d, ">=", "2014")

    # Date YYYY
    d = "2015"
    assert query_assertion(rec, d, ">=", d)
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2015-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2015-05-05T20")
    assert query_assertion(rec, d, ">=", "2015-05-05")
    assert query_assertion(rec, d, ">=", "2015-05")
    assert query_assertion(rec, d, ">=", "2015")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30.123")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15:30")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20:15")
    assert not query_assertion(rec, d, ">=", "2016-05-05T20")
    assert not query_assertion(rec, d, ">=", "2016-05-05")
    assert not query_assertion(rec, d, ">=", "2016-05")
    assert not query_assertion(rec, d, ">=", "2016")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30.123")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15:30")
    assert query_assertion(rec, d, ">=", "2014-05-05T20:15")
    assert query_assertion(rec, d, ">=", "2014-05-05T20")
    assert query_assertion(rec, d, ">=", "2014-05-05")
    assert query_assertion(rec, d, ">=", "2014-05")
    assert query_assertion(rec, d, ">=", "2014")


@pytest.mark.slow
def test_smaller_equal_operator():
    rec = set_up_rec()

    # Date YYYY-MM-ddThh:mm:ss
    d = "2015-05-05T20:15:30"
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.122")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.124")
    assert query_assertion(rec, d, "<=", d)
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T20")
    assert query_assertion(rec, d, "<=", "2015-05-05")
    assert query_assertion(rec, d, "<=", "2015-05")
    assert query_assertion(rec, d, "<=", "2015")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:15:29.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:15:29")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14:30")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15:30")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T21")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20")
    assert not query_assertion(rec, d, "<=", "2015-05-04")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-06T20")
    assert query_assertion(rec, d, "<=", "2015-05-06")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20")
    assert not query_assertion(rec, d, "<=", "2015-04-05")
    assert not query_assertion(rec, d, "<=", "2015-04")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-06-05T20")
    assert query_assertion(rec, d, "<=", "2015-06-05")
    assert query_assertion(rec, d, "<=", "2015-06")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20")
    assert not query_assertion(rec, d, "<=", "2014-05-05")
    assert not query_assertion(rec, d, "<=", "2014-05")
    assert not query_assertion(rec, d, "<=", "2014")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2016-05-05T20")
    assert query_assertion(rec, d, "<=", "2016-05-05")
    assert query_assertion(rec, d, "<=", "2016-05")
    assert query_assertion(rec, d, "<=", "2016")

    # Date YYYY-MM-ddThh:mm:ss.ns
    d = "2015-05-05T20:15:30.123"
    assert query_assertion(rec, d, "<=", d)
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T20")
    assert query_assertion(rec, d, "<=", "2015-05-05")
    assert query_assertion(rec, d, "<=", "2015-05")
    assert query_assertion(rec, d, "<=", "2015")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:15:30.122")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.124")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:15:29.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:15:29")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:31.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:31")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14:30")
    assert not query_assertion(rec, d, "<=", "2015-05-05T20:14")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:16")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15:30")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19:15")
    assert not query_assertion(rec, d, "<=", "2015-05-05T19")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T21:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T21")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20")
    assert not query_assertion(rec, d, "<=", "2015-05-04")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-06T20")
    assert query_assertion(rec, d, "<=", "2015-05-06")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20")
    assert not query_assertion(rec, d, "<=", "2015-04-05")
    assert not query_assertion(rec, d, "<=", "2015-04")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-06-05T20")
    assert query_assertion(rec, d, "<=", "2015-06-05")
    assert query_assertion(rec, d, "<=", "2015-06")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20")
    assert not query_assertion(rec, d, "<=", "2014-05-05")
    assert not query_assertion(rec, d, "<=", "2014-05")
    assert not query_assertion(rec, d, "<=", "2014")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2016-05-05T20")
    assert query_assertion(rec, d, "<=", "2016-05-05")
    assert query_assertion(rec, d, "<=", "2016-05")
    assert query_assertion(rec, d, "<=", "2016")

    # Date YYYY-MM-dd
    d = "2015-05-05"
    assert query_assertion(rec, d, "<=", d)
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T20")
    assert query_assertion(rec, d, "<=", "2015-05-05")
    assert query_assertion(rec, d, "<=", "2015-05")
    assert query_assertion(rec, d, "<=", "2015")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20:15")
    assert not query_assertion(rec, d, "<=", "2015-05-04T20")
    assert not query_assertion(rec, d, "<=", "2015-05-04")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-06T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-06T20")
    assert query_assertion(rec, d, "<=", "2015-05-06")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20")
    assert not query_assertion(rec, d, "<=", "2015-04-05")
    assert not query_assertion(rec, d, "<=", "2015-04")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-06-05T20")
    assert query_assertion(rec, d, "<=", "2015-06-05")
    assert query_assertion(rec, d, "<=", "2015-06")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20")
    assert not query_assertion(rec, d, "<=", "2014-05-05")
    assert not query_assertion(rec, d, "<=", "2014-05")
    assert not query_assertion(rec, d, "<=", "2014")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2016-05-05T20")
    assert query_assertion(rec, d, "<=", "2016-05-05")
    assert query_assertion(rec, d, "<=", "2016-05")
    assert query_assertion(rec, d, "<=", "2016")

    # Date YYYY-MM
    d = "2015-05"
    assert query_assertion(rec, d, "<=", d)
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T20")
    assert query_assertion(rec, d, "<=", "2015-05-05")
    assert query_assertion(rec, d, "<=", "2015-05")
    assert query_assertion(rec, d, "<=", "2015")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20:15")
    assert not query_assertion(rec, d, "<=", "2015-04-05T20")
    assert not query_assertion(rec, d, "<=", "2015-04-05")
    assert not query_assertion(rec, d, "<=", "2015-04")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-06-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-06-05T20")
    assert query_assertion(rec, d, "<=", "2015-06-05")
    assert query_assertion(rec, d, "<=", "2015-06")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20")
    assert not query_assertion(rec, d, "<=", "2014-05-05")
    assert not query_assertion(rec, d, "<=", "2014-05")
    assert not query_assertion(rec, d, "<=", "2014")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2016-05-05T20")
    assert query_assertion(rec, d, "<=", "2016-05-05")
    assert query_assertion(rec, d, "<=", "2016-05")
    assert query_assertion(rec, d, "<=", "2016")

    # Date YYYY
    d = "2015"
    assert query_assertion(rec, d, "<=", d)
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2015-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2015-05-05T20")
    assert query_assertion(rec, d, "<=", "2015-05-05")
    assert query_assertion(rec, d, "<=", "2015-05")
    assert query_assertion(rec, d, "<=", "2015")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30.123")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15:30")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20:15")
    assert not query_assertion(rec, d, "<=", "2014-05-05T20")
    assert not query_assertion(rec, d, "<=", "2014-05-05")
    assert not query_assertion(rec, d, "<=", "2014-05")
    assert not query_assertion(rec, d, "<=", "2014")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30.123")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15:30")
    assert query_assertion(rec, d, "<=", "2016-05-05T20:15")
    assert query_assertion(rec, d, "<=", "2016-05-05T20")
    assert query_assertion(rec, d, "<=", "2016-05-05")
    assert query_assertion(rec, d, "<=", "2016-05")
    assert query_assertion(rec, d, "<=", "2016")
